package listaDoblementeEnlazadaCircular.modelo;


public class ListaDoblementeEnlazadaCircular<T> {
	// primer elemento de la lista
	private Nodo<T> cabeza;
	// total de elementos de la lista
	private int tamanio;

	// Constructor por defecto
	public ListaDoblementeEnlazadaCircular() {
		cabeza = null;
		tamanio = 0;
	}

	/**
	 * Devuelve el tamanio de la lista
	 * 
	 * @return tamanio de la lista
	 */
	public int getTamanio() {
		return tamanio;
	}

	/**
	 * Consulta si la lista esta
	 * 
	 * @return true cuando esta vacia, false cuando no
	 */
	public boolean esVacia() {
		return (cabeza == null);
	}

	/**
	 * Agrega un nuevo nodo al final de la lista
	 * 
	 * @param valor
	 *            valor a agregar
	 */
	public void agregar(T valor) {
		Nodo<T> nuevo = new Nodo<T>();
		nuevo.setValor(valor);
		if (esVacia()) {
			cabeza = nuevo;
			// enlazar anterior y siguiente consigo mismo
			cabeza.setSiguiente(cabeza);
			cabeza.setAnterior(cabeza);
		} else {
			// agregar al final de la lista
			Nodo<T> ultimo = cabeza.getAnterior();
			ultimo.setSiguiente(nuevo);
			nuevo.setAnterior(ultimo);
			// Enlazar ultimo con primero y primero con ultimo
			nuevo.setSiguiente(cabeza);
			cabeza.setAnterior(nuevo);
		}
		tamanio++;
	}

	/**
	 * inserta un nuevo nodo en la lista
	 * 
	 * @param valor
	 *            valor a agregar
	 * @param pos
	 *            posicion donde se insertara el nodo
	 * @throws PosicionIlegalException
	 *             excepcion en caso que la posicion no exista
	 */
	public void insertar(T valor, int pos) throws PosicionIlegalException {
		// para agregar al menos debe haber un dato en la lista
		if (pos >= 0 && pos <= tamanio && tamanio > 0) {
			Nodo<T> nuevo = new Nodo<T>();
			nuevo.setValor(valor);
			// el nodo a insertar esta al comienzo de la lista
			if (pos == 0) {
				Nodo<T> ultimo = cabeza.getAnterior();
				nuevo.setSiguiente(cabeza);
				cabeza.setAnterior(nuevo);
				cabeza = nuevo;
				// enlazar primero con ultimo y ultimo con primero
				nuevo.setAnterior(ultimo);
				ultimo.setSiguiente(cabeza);
			} else {
				// el nodo a insertar va al final de la lista
				if (pos == tamanio) {
					Nodo<T> ultimo = cabeza.getAnterior();
					ultimo.setSiguiente(nuevo);
					nuevo.setAnterior(ultimo);
					// enlazar primero con ultimo y ultimo con primero
					nuevo.setSiguiente(cabeza);
					cabeza.setAnterior(nuevo);
				}
				// el nodo a insertar esta en medio
				else {
					Nodo<T> aux = cabeza;
					for (int i = 0; i <= pos - 2; i++) {
						aux = aux.getSiguiente();
					}
					Nodo<T> siguiente = aux.getSiguiente();
					aux.setSiguiente(nuevo);
					nuevo.setSiguiente(siguiente);
					nuevo.setAnterior(aux);
					siguiente.setAnterior(nuevo);
				}
			}
			tamanio++;
		} else {
			throw new PosicionIlegalException();
		}

	}

	/**
	 * Devuelve el valor de una determinada posicion
	 * 
	 * @param pos
	 *            posicion
	 * @return el valor de tipo T
	 * @throws PosicionIlegalException
	 */
	public T getValor(int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos < tamanio) {
			T valor;
			if (pos == 0) {
				valor = cabeza.getValor();
				return valor;
			} else {
				Nodo<T> aux = cabeza;
				for (int i = 0; i <= pos - 1; i++) {
					aux = aux.getSiguiente();
				}
				valor = aux.getValor();
			}
			return valor;
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Elimina un nodo en una determinada posicion
	 * 
	 * @param pos
	 *            posicionSystem.out.println("");
	 * @throws PosicionIlegalException
	 */
	public void remover(int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos < tamanio) {
			if (pos == 0) {
				// el nodo a eliminar esta en la primera posicion
				Nodo<T> ultimo = cabeza.getAnterior();
				cabeza = cabeza.getSiguiente();
				// enlazar primero con ultimo y ultimo con primero
				cabeza.setAnterior(ultimo);
				ultimo.setSiguiente(cabeza);
			} else {
				Nodo<T> aux = cabeza;
				for (int i = 0; i <= pos - 2; i++) {
					aux = aux.getSiguiente();
				}
				Nodo<T> prox = aux.getSiguiente();
				prox.getSiguiente().setAnterior(aux);
				aux.setSiguiente(prox.getSiguiente());
			}
			tamanio--;
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Imprime todos los nodos, sirve para probar que funciona bien
	 */

	public void imprimirTodo() {
		Nodo<T> aux = cabeza;
		System.out.println("Listado de todos los nodos:");
		Object nodo = new Object();
		while (aux.getSiguiente() != cabeza) {
			nodo = aux.getValor();
			System.out.println("" + nodo.toString());
			aux = aux.getSiguiente();
		}
		nodo = aux.getValor();
		System.out.println("" + nodo.toString());
		System.out.println("De atras para adelante:");
		
		// De atras para adelante
		System.out.println("De atras para adelante:");
		Nodo<T> ultimo = cabeza.getAnterior();
		aux = ultimo;
		while (aux.getAnterior() != ultimo) {
			nodo = aux.getValor();
			System.out.println("" + nodo.toString());
			aux = aux.getAnterior();
		}
		nodo = aux.getValor();
		System.out.println("" + nodo.toString());
		
	}

	/**
	 * Elimina todos los nodos de la lista
	 */
	public void limpiar() {
		cabeza = null;
		tamanio = 0;
	}
}
